@@ -13,35 +13,35 @@ Options:
     -h, --help      print short, long help
 
 Key Binding:
-    Help            : ?
-    Quit            : q
-    Scroll down     : DOWN      j
-    Scroll up       : UP        k
-    Half screen up  : C-u
-    Half screen dn  " C-d
-    Page down       : PGDN      RIGHT   SPC
-    Page up         : PGUP      LEFT
-    Next chapter    : n
-    Prev chapter    : p
-    Beginning of ch : HOME      g
-    End of ch       : END       G
-    Open image      : o
-    Search          : /
-    Next Occurence  : n
-    Prev Occurence  : N
-    Toggle width    : =
-    Set width       : [count]=
-    Shrink          : -
-    Enlarge         : +
-    ToC             : TAB       t
-    Metadata        : m
-    Mark pos to n   : b[n]
-    Jump to pos n   : `[n]
-    Switch colorsch : [default=0, dark=1, light=2]c
+    Help             : ?
+    Quit             : q
+    Scroll down      : DOWN      j
+    Scroll up        : UP        k
+    Half screen up   : C-u
+    Half screen dn   : C-d
+    Page down        : PGDN      RIGHT   SPC
+    Page up          : PGUP      LEFT
+    Next chapter     : n
+    Prev chapter     : p
+    Beginning of ch  : HOME      g
+    End of ch        : END       G
+    Open image       : o
+    Search           : /
+    Next Occurrence  : n
+    Prev Occurrence  : N
+    Toggle width     : =
+    Set width        : [count]=
+    Shrink           : -
+    Enlarge          : +
+    ToC              : TAB       t
+    Metadata         : m
+    Mark pos to n    : b[n]
+    Jump to pos n    : `[n]
+    Switch colorsch  : [default=0, dark=1, light=2]c
 """
 
 
-__version__ = "2.4.8"
+__version__ = "2.4.15"
 __license__ = "MIT"
 __author__ = "Benawi Adha"
 __email__ = "benawiadha@gmail.com"
@@ -66,8 +66,10 @@ from difflib import SequenceMatcher as SM
 
 
 # key bindings
-SCROLL_DOWN = {curses.KEY_DOWN, ord("j")}
-SCROLL_UP = {curses.KEY_UP, ord("k")}
+SCROLL_DOWN = {curses.KEY_DOWN}
+SCROLL_DOWN_J = {ord("j")}
+SCROLL_UP = {curses.KEY_UP}
+SCROLL_UP_K = {ord("k")}
 HALF_DOWN = {4}
 HALF_UP = {21}
 PAGE_DOWN = {curses.KEY_NPAGE, ord("l"), ord(" "), curses.KEY_RIGHT}
@@ -242,9 +244,13 @@ class HTMLtoLines(HTMLParser):
             self.text[-1] += "^{"
         elif tag == "sub":
             self.text[-1] += "_{"
-        elif tag == "image":
+        # NOTE: "img" and "image"
+        # In HTML, both are startendtag (no need endtag)
+        # but in XHTML both need endtag
+        elif tag in {"img", "image"}:
             for i in attrs:
-                if i[0] == "xlink:href":
+                if (tag == "img" and i[0] == "src")\
+                   or (tag == "image" and i[0].endswith("href")):
                     self.text.append("[IMG:{}]".format(len(self.imgs)))
                     self.imgs.append(unquote(i[1]))
 
@@ -253,7 +259,8 @@ class HTMLtoLines(HTMLParser):
             self.text += [""]
         elif tag in {"img", "image"}:
             for i in attrs:
-                if (tag == "img" and i[0] == "src") or (tag == "image" and i[0] == "xlink:href"):
+                if (tag == "img" and i[0] == "src")\
+                   or (tag == "image" and i[0].endswith("href")):
                     self.text.append("[IMG:{}]".format(len(self.imgs)))
                     self.imgs.append(unquote(i[1]))
                     self.text.append("")
@@ -281,7 +288,7 @@ class HTMLtoLines(HTMLParser):
             self.isbull = False
         elif tag in {"sub", "sup"}:
             self.text[-1] += "}"
-        elif tag == "image":
+        elif tag in {"img", "image"}:
             self.text.append("")
 
     def handle_data(self, raw):
@@ -431,11 +438,11 @@ def toc(stdscr, src, index):
         if key_toc in range(48, 58): # i.e., k is a numeral
             countstring = countstring + chr(key_toc)
         else:
-            if key_toc in SCROLL_UP or key_toc in PAGE_UP:
+            if key_toc in SCROLL_UP|SCROLL_UP_K or key_toc in PAGE_UP:
                 index -= count
                 if index < 0:
                     index = 0
-            elif key_toc in SCROLL_DOWN or key_toc in PAGE_DOWN:
+            elif key_toc in SCROLL_DOWN|SCROLL_DOWN_J or key_toc in PAGE_DOWN:
                 index += count
                 if index + 1 >= totlines:
                     index = totlines - 1
@@ -515,9 +522,9 @@ def meta(stdscr, ebook):
     padhi = rows - 5 - Y - 4 + 1
 
     while key_meta not in META|QUIT:
-        if key_meta in SCROLL_UP and y > 0:
+        if key_meta in SCROLL_UP|SCROLL_UP_K and y > 0:
             y -= 1
-        elif key_meta in SCROLL_DOWN and y < totlines - hi + 6:
+        elif key_meta in SCROLL_DOWN|SCROLL_DOWN_J and y < totlines - hi + 6:
             y += 1
         elif key_meta in PAGE_UP:
             y = pgup(y, padhi)
@@ -569,9 +576,9 @@ def help(stdscr):
     padhi = rows - 5 - Y - 4 + 1
 
     while key_help not in HELP|QUIT:
-        if key_help in SCROLL_UP and y > 0:
+        if key_help in SCROLL_UP|SCROLL_UP_K and y > 0:
             y -= 1
-        elif key_help in SCROLL_DOWN and y < totlines - hi + 6:
+        elif key_help in SCROLL_DOWN|SCROLL_DOWN_J and y < totlines - hi + 6:
             y += 1
         elif key_help in PAGE_UP:
             y = pgup(y, padhi)
@@ -667,7 +674,10 @@ def searching(stdscr, pad, src, width, y, ch, tot):
         stat.addstr(0, 7, SEARCHPATTERN)
         stat.refresh()
         while True:
-            ipt = stat.getch()
+            ipt = stat.get_wch()
+            if type(ipt) == str:
+                ipt = ord(ipt)
+
             if ipt == 27:
                 stat.clear()
                 stat.refresh()
@@ -683,7 +693,7 @@ def searching(stdscr, pad, src, width, y, ch, tot):
                 curses.curs_set(0)
                 break
             # TODO: why different behaviour unix dos or win lin
-            elif ipt in {8, curses.KEY_BACKSPACE}:
+            elif ipt in {8, 127, curses.KEY_BACKSPACE}:
                 SEARCHPATTERN = SEARCHPATTERN[:-1]
             elif ipt == curses.KEY_RESIZE:
                 stat.clear()
@@ -909,6 +919,11 @@ def reader(stdscr, ebook, index, width, y, pctg):
                     return -1, width, -rows, None
                 else:
                     y = 0
+            elif k in SCROLL_UP_K:
+                if count > 1:
+                    svline = y - 1
+                if y >= count:
+                    y -= count
             elif k in PAGE_UP:
                 if y == 0 and index != 0:
                     return -1, width, -rows, None
@@ -923,6 +938,11 @@ def reader(stdscr, ebook, index, width, y, pctg):
                     return 1, width, 0, None
                 else:
                     y = totlines - rows
+            elif k in SCROLL_DOWN_J:
+                if count > 1:
+                    svline = y + rows - 1
+                if y + count <= totlines - rows:
+                    y += count
             elif k in PAGE_DOWN:
                 if totlines - y - LINEPRSRV > rows:
                     # y = pgdn(y, totlines, rows, LINEPRSRV, count)
